@@ -2,7 +2,7 @@
 /* Mednafen - Multi-system Emulator                                           */
 /******************************************************************************/
 /* FileStream.h:
-**  Copyright (C) 2010-2016 Mednafen Team
+**  Copyright (C) 2010-2021 Mednafen Team
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License
@@ -33,11 +33,11 @@ class FileStream : public Stream
  public:
 
  // Convenience function so we don't need so many try { } catch { } for ENOENT
- static INLINE FileStream* open(const std::string& path, const uint32 mode, const int do_lock = false)
+ static INLINE FileStream* open(const std::string& path, const uint32 mode, const int do_lock = false, const uint32 buffer_size = 4096)
  {
   try
   {
-   return new FileStream(path, mode, do_lock);
+   return new FileStream(path, mode, do_lock, buffer_size);
   }
   catch(MDFN_Error& e)
   {
@@ -59,7 +59,7 @@ class FileStream : public Stream
   MODE_WRITE_INPLACE = VirtualFS::MODE_WRITE_INPLACE,	// Like MODE_WRITE, but won't truncate the file if it already exists.
  };
 
- FileStream(const std::string& path, const uint32 mode, const int do_lock = false);
+ FileStream(const std::string& path, const uint32 mode, const int do_lock = false, const uint32 buffer_size = 4096);
  virtual ~FileStream() override;
 
  virtual uint64 attributes(void) override;
@@ -81,40 +81,65 @@ class FileStream : public Stream
 
  INLINE int get_char(void)
  {
-  int ret;
+  unsigned char ret;
 
-  if(MDFN_UNLIKELY(prev_was_write == 1))
-   seek(0, SEEK_CUR);
-
-  errno = 0;
-  ret = fgetc(fp);
-
-  if(MDFN_UNLIKELY(errno != 0))
+  if(buf_read_offs < buf_read_avail)
   {
-   ErrnoHolder ene(errno);
-   throw(MDFN_Error(ene.Errno(), _("Error reading from opened file \"%s\": %s"), path_save.c_str(), ene.StrError()));
+   ret = buf[buf_read_offs++];
+   pos++;
   }
-  return(ret);
+  else if(!read(&ret, 1, false))
+   return -1;
+
+  return ret;
  }
 
+ INLINE void put_char(char c)
+ {
+  if(!buf_read_avail && (OpenedMode != MODE_READ) && buf_write_offs < buf_size)
+  {
+   buf[buf_write_offs] = c;
+   buf_write_offs++;
+
+   if(buf_write_offs == buf_size)
+    write_buffered_data();
+  }
+  else
+   write(&c, 1);
+ }
+
+ void set_buffer_size(uint32 new_size);
+
  private:
  FileStream & operator=(const FileStream &);    // Assignment operator
  FileStream(const FileStream &);		// Copy constructor
  //FileStream(FileStream &);                // Copy constructor
 
- FILE *fp;
- std::string path_save;
- const uint32 OpenedMode;
+ void lock(bool nb);
+ void unlock(void);
+
+ uint64 read_ub(void* data, uint64 count);
+ uint64 write_ub(const void* data, uint64 count);
+ void write_buffered_data(void);
+
+ int fd;
+ uint64 pos;
+
+ std::unique_ptr<uint8[]> buf;
+ uint32 buf_size;
+ uint32 buf_write_offs;
+ uint32 buf_read_offs;
+ uint32 buf_read_avail;
+
+ bool need_real_seek;
+
+ uint64 locked;
 
  void* mapping;
  uint64 mapping_size;
 
- bool locked;
- int prev_was_write;	// -1 for no state, 0 for last op was read, 1 for last op was write(used for MODE_READ_WRITE)
-
- //
- void lock(bool nb);
- void unlock(void);
+ const uint32 OpenedMode;
+ const std::string path_humesc;
 };
 
 }
