@@ -2,7 +2,7 @@
 /* Mednafen - Multi-system Emulator                                           */
 /******************************************************************************/
 /* Stream.h:
-**  Copyright (C) 2012-2018 Mednafen Team
+**  Copyright (C) 2012-2021 Mednafen Team
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License
@@ -214,6 +214,13 @@ class Stream
   write(str, strlen(str));
  }
 
+ uint64 get_string_append(std::string* str, uint64 count, bool error_on_eos = true);
+ INLINE uint64 get_string(std::string* str, uint64 count, bool error_on_eos = true)
+ {
+  str->clear();
+  return get_string_append(str, count, error_on_eos);
+ }
+
  // Reads a line into "str", overwriting its contents; returns the line-end char('\n' or '\r' or '\0'), or 256 on EOF and
  // data has been read into "str", and -1 on EOF when no data has been read into "str".
  // The line-end char won't be added to "str".
@@ -242,7 +249,7 @@ class Stream
  // Read until end-of-stream(or count), discarding any read data, and returns the amount of data "read".
  //  (Useful for detecting and printing warnings about extra garbage data without needing to call size(),
  //   which can be problematic for some types of Streams).
- uint64 read_discard(uint64 count = ~(uint64)0);
+ uint64 read_discard(uint64 count = (uint64)-1);
 
  //
  // Reads stream starting at the current stream position(as returned by tell()), into memory allocated with malloc() and realloc(), and
@@ -256,7 +263,7 @@ class Stream
  //
  // If the returned value is 0, *data_out will still be a valid non-NULL pointer.
  //
- uint64 alloc_and_read(void** data_out, uint64 size_limit = ~(uint64)0);
+ uint64 alloc_and_read(void** data_out, uint64 size_limit = (uint64)-1);
 };
 
 //
