@@ -87,7 +87,7 @@ static const MDFNSetting_EnumList SDriver_List[] =
 
  { "alsa", -1, "ALSA", gettext_noop("The default for Linux(if available).") },
  { "openbsd", -1, "OpenBSD Audio", gettext_noop("The default for OpenBSD.") },
- { "oss", -1, "Open Sound System", gettext_noop("The default for non-Linux UN*X/POSIX/BSD(other than OpenBSD) systems, or anywhere ALSA is unavailable. If the ALSA driver gives you problems, you can try using this one instead.\n\nIf you are using OSSv4 or newer, you should edit \"/usr/lib/oss/conf/osscore.conf\", uncomment the max_intrate= line, and change the value from 100(default) to 1000(or higher if you know what you're doing), and restart OSS. Otherwise, performance will be poor, and the sound buffer size in Mednafen will be orders of magnitude larger than specified.\n\nIf the sound buffer size is still excessively larger than what is specified via the \"sound.buffer_time\" setting, you can try setting \"sound.period_time\" to 2666, and as a last resort, 5333, to work around a design flaw/limitation/choice in the OSS API and OSS implementation.") },
+ { "oss", -1, "Open Sound System", gettext_noop("The default for non-Linux UN*X/POSIX/BSD(other than OpenBSD) systems, or anywhere ALSA is unavailable. If the ALSA driver gives you problems, you can try using this one instead.\n\nIf you are using OSSv4 or newer, you should edit \"/usr/lib/oss/conf/osscore.conf\", uncomment the max_intrate= line, and change the value from 100(default) to 1000(or higher if you know what you're doing), and restart OSS. Otherwise, performance will be poor, and the sound buffer size in Mednafen will be orders of magnitude larger than specified.\n\nIf the sound buffer size is still excessively larger than what is specified via the \"\5sound.buffer_time\" setting, you can try setting \"\5sound.period_time\" to 2666, and as a last resort, 5333, to work around a design flaw/limitation/choice in the OSS API and OSS implementation.") },
 
  { "wasapish", -1, "WASAPI(Shared Mode)", gettext_noop("The default when it's available(running on Microsoft Windows Vista and newer).") },
 
@@ -138,6 +138,17 @@ static const MDFNSetting_EnumList FPSPos_List[] =
  { NULL, 0 },
 };
 
+static const MDFNSetting_EnumList InputGrabStrat_List[] =
+{
+ { "auto", INPUTGRAB_STRAT_AUTO, gettext_noop("Auto."), gettext_noop("Grab the system mouse only when an emulated mouse or other device that requires relative ball motion is active, and grab the system keyboard only when an emulated keyboard is active.") },
+ { "full", INPUTGRAB_STRAT_FULL, gettext_noop("Full."), gettext_noop("Grab the system mouse and keyboard regardless of whether or not an emulated keyboard or mouse/ball device is active.") },
+
+ { "kb_auto",	INPUTGRAB_STRAT_KB_AUTO, gettext_noop("Keyboard auto, mouse regardless of emulated devices.") },
+ { "mouse_auto",INPUTGRAB_STRAT_MOUSE_AUTO, gettext_noop("Mouse auto, keyboard regardless of emulated devices.") },
+
+ { NULL, 0 }
+};
+
 static const MDFNSetting DriverSettings[] =
 {
   { "input.joystick.global_focus", MDFNSF_NOFLAGS, gettext_noop("Update physical joystick(s) internal state in Mednafen even when Mednafen lacks OS focus."), NULL, MDFNST_BOOL, "1" },
@@ -145,6 +156,8 @@ static const MDFNSetting DriverSettings[] =
   { "input.autofirefreq", MDFNSF_NOFLAGS, gettext_noop("Auto-fire frequency."), gettext_noop("Auto-fire frequency = GameSystemFrameRateHz / (value + 1)"), MDFNST_UINT, "3", "0", "1000" },
   { "input.ckdelay", MDFNSF_NOFLAGS, gettext_noop("Dangerous key action delay."), gettext_noop("The length of time, in milliseconds, that a button/key corresponding to a \"dangerous\" command like power, reset, exit, etc. must be pressed before the command is executed."), MDFNST_UINT, "0", "0", "99999" },
 
+  { "input.grab.strategy", MDFNSF_NOFLAGS, gettext_noop("Input grabbing strategy."), gettext_noop("Selects the conditions for and extent of system keyboard and mouse grabbing when input grabbing is toggled on.\n\nNote that regardless of this setting, system keyboard grabbing is temporarily disabled while in the cheat interface, debugger, or netplay console text entry."), MDFNST_ENUM, "full", NULL, NULL, NULL, NULL, InputGrabStrat_List },
+
   { "netplay.host", MDFNSF_NOFLAGS, gettext_noop("Server hostname."), NULL, MDFNST_STRING, "netplay.fobby.net" },
   { "netplay.port", MDFNSF_NOFLAGS, gettext_noop("Server port."), NULL, MDFNST_UINT, "4046", "1", "65535" },
   { "netplay.console.font", MDFNSF_NOFLAGS, gettext_noop("Font for netplay chat console."), NULL, MDFNST_ENUM, "9x18", NULL, NULL, NULL, NULL, FontSize_List },
@@ -171,7 +184,7 @@ static const MDFNSetting DriverSettings[] =
   { "sound.device", MDFNSF_NOFLAGS, gettext_noop("Select sound output device."), gettext_noop("When using ALSA sound output under Linux, the \"sound.device\" setting \"default\" is Mednafen's default, IE \"hw:0\", not ALSA's \"default\". If you want to use ALSA's \"default\", use \"sexyal-literal-default\"."), MDFNST_STRING, "default", NULL, NULL },
   { "sound.volume", MDFNSF_NOFLAGS, gettext_noop("Sound volume level, in percent."), gettext_noop("Setting this volume control higher than the default of \"100\" may severely distort the sound."), MDFNST_UINT, "100", "0", "150" },
   { "sound", MDFNSF_NOFLAGS, gettext_noop("Enable sound output."), NULL, MDFNST_BOOL, "1" },
-  { "sound.period_time", MDFNSF_NOFLAGS, gettext_noop("Desired period size in microseconds(μs)."), gettext_noop("Currently only affects OSS, ALSA, WASAPI(exclusive mode), and SDL output.  A value of 0 defers to the default in the driver code in SexyAL.\n\nNote: This is not the \"sound buffer size\" setting, that would be \"sound.buffer_time\"."), MDFNST_UINT,  "0", "0", "100000" },
+  { "sound.period_time", MDFNSF_NOFLAGS, gettext_noop("Desired period size in microseconds(μs)."), gettext_noop("Currently only affects OSS, ALSA, WASAPI(exclusive mode), and SDL output.  A value of 0 defers to the default in the driver code in SexyAL.\n\nNote: This is not the \"sound buffer size\" setting, that would be \"\5sound.buffer_time\"."), MDFNST_UINT,  "0", "0", "100000" },
   { "sound.buffer_time", MDFNSF_NOFLAGS, gettext_noop("Desired buffer size in milliseconds(ms)."), gettext_noop("The default value of 0 enables automatic buffer size selection."), MDFNST_UINT, "0", "0", "1000" },
   { "sound.rate", MDFNSF_NOFLAGS, gettext_noop("Specifies the sound playback rate, in sound frames per second(\"Hz\")."), NULL, MDFNST_UINT, "48000", "22050", "192000"},
 
@@ -269,8 +282,6 @@ static MThreading::Mutex *StdoutMutex = NULL;
 //
 //
 
-static bool sc_blit_timesync;
-
 static char *soundrecfn=0;	/* File name of sound recording. */
 
 static char *qtrecfn = NULL;
@@ -524,9 +535,9 @@ static bool CreateDataDirs(void)
    std::string dir = MDFN_GetSettingS(s.sname_path);
 
    if(!NVFS.is_absolute_path(dir))
-    dir = DrBaseDirectory + PSS + dir;
+    dir = DrBaseDirectory + MDFN_PSS + dir;
 
-   NVFS.create_missing_dirs(dir + PSS);
+   NVFS.create_missing_dirs(dir + MDFN_PSS);
   }
  }
  catch(std::exception &e)
@@ -597,7 +608,7 @@ static SignalInfo SignalDefs[] =
  #endif
 };
 
-static volatile int SignalSTDOUT;
+static int volatile SignalSTDOUT;
 
 static void SetSignals(void (*t)(int))
 {
@@ -817,13 +828,27 @@ static MDFN_COLD void PrintConfigMacros(void)
  MACROSTR(MDFN_HOT)
  MACROSTR(MDFN_ASSUME_ALIGNED(p, align))
  MACROSTR(MDFN_HIDE)
- MACROSTR(PSS_STYLE)
+ MACROSTR(MDFN_PSS_STYLE)
 #endif
  //
  #undef MACROSTR
  #undef MACROSTR2
 }
 
+static int DoArgs_LoadOverrideConfig(const char *name, const char *value)
+{
+ int r = MDFNI_LoadSettings(value, true);
+
+ if(r < 0)
+ {
+  ErrnoHolder ene(ENOENT);
+
+  MDFN_Notify(MDFN_NOTICE_ERROR, _("Failed to load settings from \"%s\": %s"), MDFN_strhumesc(value).c_str(), ene.StrError());
+ }
+
+ return r > 0;
+}
+
 static int netconnect = 0;
 static char* loadcd = NULL;	// Deprecated
 static int which_medium = -2;
@@ -868,7 +893,8 @@ static bool DoArgs(int argc, char *argv[], char **filename)
         ARGPSTRUCT MDFNArgs[] = 
 	{
 	 { "help", _("Show help!"), &ShowCLHelp, 0, 0 },
-	 { "remote", _("Enable remote mode with the specified stdout key(EXPERIMENTAL AND INCOMPLETE)."), 0, &dummy_remote, SUBSTYPE_STRING_ALLOC },
+
+	 { "ovconfig", _("Load global override settings from specified file."), NULL, (void*)DoArgs_LoadOverrideConfig, SUBSTYPE_FUNCTION },
 
 	 // -loadcd is deprecated and only still supported because it's been around for yeaaaars.
 	 { "loadcd", NULL/*_("Load and boot a CD for the specified system.")*/, 0, &loadcd, SUBSTYPE_STRING_ALLOC },
@@ -880,8 +906,9 @@ static bool DoArgs(int argc, char *argv[], char **filename)
 	 { "soundrecord", _("Record sound output to the specified filename in the MS WAV format."), 0,&soundrecfn, SUBSTYPE_STRING_ALLOC },
 	 { "qtrecord", _("Record video and audio output to the specified filename in the QuickTime format."), 0, &qtrecfn, SUBSTYPE_STRING_ALLOC }, // TODOC: Video recording done without filtering applied.
 
-	 { "dump_settings_def", _("Dump settings definition data to specified file."), 0, &dsfn, SUBSTYPE_STRING_ALLOC },
-	 { "dump_modules_def", _("Dump modules definition data to specified file."), 0, &dmfn, SUBSTYPE_STRING_ALLOC },
+	 { "remote", /*_("Enable remote mode with the specified stdout key(EXPERIMENTAL AND INCOMPLETE).")*/NULL, 0, &dummy_remote, SUBSTYPE_STRING_ALLOC },
+	 { "dump_settings_def", /*_("Dump settings definition data to specified file.")*/NULL, 0, &dsfn, SUBSTYPE_STRING_ALLOC },
+	 { "dump_modules_def", /*_("Dump modules definition data to specified file.")*/NULL, 0, &dmfn, SUBSTYPE_STRING_ALLOC },
 
          { 0, NULL, (int *)InternalArgs.get(), 0, 0},
 
@@ -1003,19 +1030,30 @@ static bool DoArgs(int argc, char *argv[], char **filename)
 	return true;
 }
 
-static volatile unsigned NeedVideoSync = 0;
-static int GameLoop(void *arg);
-int volatile GameThreadRun = 0;
+static void CloseGame(void) MDFN_COLD;
+//
+//
+static unsigned volatile NeedVideoSync = false;
+static int volatile NeedExitNow = false;	// Set 'true' in various places, including signal handler.
+//
+//
 static bool MDFND_Update(int WhichVideoBuffer, int16 *Buffer, int Count);
+static void GameThread_HandleEvents(void);
+static int GameLoop(void *arg);
+static int volatile GameThreadRun = false;
+static bool InFrameAdvance = false;
+static bool NeedFrameAdvance = false;
+static bool GameLoopPaused = false;
+double CurGameSpeed = 1;
+
+static bool sc_blit_timesync;
 
 static bool sound_active;	// true if sound is enabled and initialized
 
-
 static EmuRealSyncher ers;
 
 static bool autosave_load_error = false;
 
-static void CloseGame(void) MDFN_COLD;
 static int LoadGame(const char *force_module, const char *path)
 {
 	assert(MThreading::Thread_ID() == MainThreadID);
@@ -1154,7 +1192,7 @@ static void CloseGame(void)
 	//
 	//
 	//
-	GameThreadRun = 0;
+	GameThreadRun = false;
 
 	if(GameThread)
 	{
@@ -1187,23 +1225,16 @@ static void CloseGame(void)
 	CurGame = NULL;
 }
 
-static void GameThread_HandleEvents(void);
-static int volatile NeedExitNow = 0;	// Set 'true' in various places, including signal handler.
-double CurGameSpeed = 1;
-
 void MainRequestExit(void)
 {
- NeedExitNow = 1;
+ NeedExitNow = true;
 }
 
 bool Mednafen::MDFND_CheckNeedExit(void)	// Called from netplay code, so we can break out of blocking loops after receiving a signal.
 {
- return (bool)NeedExitNow;
+ return !GameThreadRun;
 }
 
-static bool InFrameAdvance = 0;
-static bool NeedFrameAdvance = 0;
-
 bool IsInFrameAdvance(void)
 {
  return InFrameAdvance;
@@ -1221,10 +1252,12 @@ void DoFrameAdvance(void)
  InFrameAdvance = 1;
 }
 
-static int GameLoopPaused = 0;
-
-void DebuggerFudge(void)
+bool DebuggerFudge(void)
 {
+ if(!GameThreadRun)
+  return false;
+ //
+ //
  const uint32 WaitMS = 10;
  uint32 wt = Time::MonoMS() + WaitMS;
 
@@ -1239,21 +1272,26 @@ void DebuggerFudge(void)
   else
    Time::SleepMS(wt);
  }
+
+ return true;
 }
 
 static int GameLoop(void *arg)
 {
 	while(GameThreadRun)
 	{
-         int16 *sound;
-         int32 ssize;
-         bool fskip;
+	 int16 *sound;
+	 int32 ssize;
+	 uint32 mcycs;
+	 bool fskip;
         
 	 /* If we requested a new video mode, wait until it's set before calling the emulation code again.
 	 */
 	 while(NeedVideoSync)
 	 {
-	  if(!GameThreadRun) return(1);	// Might happen if video initialization failed
+	  if(!GameThreadRun)
+	   return 1;		// Might happen if video initialization failed
+
 	  Time::SleepMS(2);
 	 }
 
@@ -1363,13 +1401,12 @@ static int GameLoop(void *arg)
 	  }
 	 }
 
-	 ers.AddEmuTime((espec.MasterCycles - espec.MasterCycles_DriverProcessed) / CurGameSpeed);
-
 	 SoftFB[SoftFB_BackBuffer].rect = espec.DisplayRect;
 	 SoftFB[SoftFB_BackBuffer].field = espec.InterlaceOn ? espec.InterlaceField : -1;
 
 	 sound = espec.SoundBuf + (espec.SoundBufSize_DriverProcessed * CurGame->soundchan);
 	 ssize = espec.SoundBufSize - espec.SoundBufSize_DriverProcessed;
+	 mcycs = (espec.MasterCycles - espec.MasterCycles_DriverProcessed) / CurGameSpeed;
 	 //
 	 //
 	 //
@@ -1384,6 +1421,8 @@ static int GameLoop(void *arg)
 
 	  do
 	  {
+	   ers.AddEmuTime(mcycs);	// Call before MDFND_Update()
+
  	   if(fskip && ((InFrameAdvance && !NeedFrameAdvance) || GameLoopPaused))
 	   {
 	    // If this frame was skipped, and the game loop is paused(IE cheat interface is active) or we're in frame advance, just blit the last
@@ -1406,16 +1445,24 @@ static int GameLoop(void *arg)
 
            if((InFrameAdvance && !NeedFrameAdvance) || GameLoopPaused)
 	   {
-            if(ssize)
-	     for(int x = 0; x < CurGame->soundchan * ssize; x++)
-	      sound[x] = 0;
+	    int32 sbms;
+
+	    mcycs = CurGame->MasterClock / MDFN_MASTERCLOCK_FIXED(50);
+	    sound = Sound_GetEmuModBuffer(&sbms);
+	    ssize = std::min<int32>(sbms, Sound_GetRate() / 50);
+
+	    if(sound)
+	    {
+	     for(int i = 0; i < CurGame->soundchan * ssize; i++)
+	      sound[i] = 0;
+	    }
 	   }
 	  } while(((InFrameAdvance && !NeedFrameAdvance) || GameLoopPaused) && GameThreadRun);
 	  SoftFB_BackBuffer ^= do_flip;
 	 }
 	}
 
-	return(1);
+	return 1;
 }   
 
 
@@ -1431,7 +1478,7 @@ std::string GetBaseDirectory(void)
 
   ol = _tgetenv(TEXT("HOME"));
   if(ol)
-   return Win32Common::T_to_UTF8(ol, nullptr, true) + PSS + ".mednafen";
+   return Win32Common::T_to_UTF8(ol, nullptr, true) + MDFN_PSS + ".mednafen";
  }
 
  {
@@ -1456,7 +1503,7 @@ std::string GetBaseDirectory(void)
 
  ol = getenv("HOME");
  if(ol)
-  return std::string(ol) + PSS + ".mednafen";
+  return std::string(ol) + MDFN_PSS + ".mednafen";
 
  #if defined(HAVE_GETUID) && defined(HAVE_GETPWUID)
  {
@@ -1465,7 +1512,7 @@ std::string GetBaseDirectory(void)
   psw = getpwuid(getuid());
 
   if(psw != NULL && psw->pw_dir[0] != 0 && strcmp(psw->pw_dir, "/dev/null"))
-   return std::string(psw->pw_dir) + PSS + ".mednafen";
+   return std::string(psw->pw_dir) + MDFN_PSS + ".mednafen";
  }
  #endif
  return "";
@@ -1496,8 +1543,7 @@ static void GameThread_HandleEvents(void)
 		switch(event.user.code & 0xFFFF)
 		{
 		 case CEVT_SET_INPUT_FOCUS:
-			MDFNDHaveFocus = (event.user.data1 != NULL);
-			//printf("%u\n", MDFNDHaveFocus);
+			Input_SetFocus((event.user.data1 != NULL));
 			break;
 		}
 		break;
@@ -1545,7 +1591,7 @@ void GT_ToggleFS(void)
  // assert(MThreading::Thread_ID() == GameThreadID);
  MThreading::Mutex_Lock(VTMutex);
  MDFNI_SetSettingB("video.fs", !MDFN_GetSettingB("video.fs"));
- NeedVideoSync++;
+ NeedVideoSync = true;
  MThreading::Mutex_Unlock(VTMutex);
 
  MThreading::Sem_Post(VTWakeupSem);
@@ -1559,7 +1605,7 @@ bool GT_ReinitVideo(void)
 {
  // assert(MThreading::Thread_ID() == GameThreadID);
  MThreading::Mutex_Lock(VTMutex);
- NeedVideoSync++;
+ NeedVideoSync = true;
  MThreading::Mutex_Unlock(VTMutex);
 
  MThreading::Sem_Post(VTWakeupSem);
@@ -1588,9 +1634,9 @@ bool GT_ReinitSound(void)
  return(ret);
 }
 
-void GT_SetWMInputBehavior(bool CursorNeeded, bool MouseAbsNeeded, bool MouseRelNeeded, bool GrabNeeded)
+void GT_SetWMInputBehavior(const WMInputBehavior& wmib)
 {
- SendCEvent(CEVT_SET_WMINPUTBEHAVIOR, nullptr, nullptr, (CursorNeeded << 0) | (MouseAbsNeeded << 1) | (MouseRelNeeded << 2) | (GrabNeeded << 3));
+ SendCEvent(CEVT_SET_WMINPUTBEHAVIOR, nullptr, nullptr, (wmib.Cursor << 0) | (wmib.MouseAbs << 1) | (wmib.MouseRel << 2) | (wmib.Grab_Keyboard << 3) | (wmib.Grab_Mouse << 4));
 }
 
 void PumpWrap(void)
@@ -1633,7 +1679,7 @@ void PumpWrap(void)
 	break;
 
    case SDL_QUIT:
-	NeedExitNow = 1;
+	NeedExitNow = true;
 	break;
 
    case SDL_USEREVENT:
@@ -1650,20 +1696,25 @@ void PumpWrap(void)
 		MT_SetMovieStatus((StateStatusStruct *)event.user.data1);
 		break;
 
+	 case CEVT_CLOSE_POPUP:
+		Netplay_TryTextExit();
+		break;
+
 	 case CEVT_WANT_EXIT:
+		NeedExitNow = true;
+		break;
+
+	 case CEVT_WANT_OLD_EXIT:
 		if(!Netplay_TryTextExit())
-		{
-		 SDL_Event evt;
-		 evt.quit.type = SDL_QUIT;
-		 SDL_PushEvent(&evt);
-		}
+		 NeedExitNow = true;
 		break;
 
 	 case CEVT_SET_WMINPUTBEHAVIOR:
 		NeededWMInputBehavior.Cursor = (bool)(idata16 & 0x1);
 		NeededWMInputBehavior.MouseAbs = (bool)(idata16 & 0x2);
 		NeededWMInputBehavior.MouseRel = (bool)(idata16 & 0x4);
-		NeededWMInputBehavior.Grab = (bool)(idata16 & 0x8);
+		NeededWMInputBehavior.Grab_Keyboard = (bool)(idata16 & 0x8);
+		NeededWMInputBehavior.Grab_Mouse = (bool)(idata16 & 0x10);
 		NeededWMInputBehavior_Dirty = true;
 		break;
 
@@ -1730,7 +1781,7 @@ void PrintGLIBCXXInfo(void)
 }
 #endif
 
-void PrintSDLVersion(void)
+static bool CheckPrintSDLVersion(void)
 {
  SDL_version sver;
 
@@ -1741,8 +1792,10 @@ void PrintSDLVersion(void)
  if(SDL_VERSIONNUM(sver.major, sver.minor, sver.patch) < SDL_VERSIONNUM(SDL_MAJOR_VERSION, SDL_MINOR_VERSION, SDL_PATCHLEVEL))
  {
   MDFN_Notify(MDFN_NOTICE_ERROR, _("Running with a version of SDL older than the version compiled and linked against!"));
-  abort();
+  return false;
  }
+
+ return true;
 }
 
 #ifdef HAVE_LIBFLAC
@@ -1828,8 +1881,8 @@ char *GetFileDialog(void)
 
 static bool LoadSettings(void)
 {
- const std::string opath09x = DrBaseDirectory + PSS + "mednafen-09x.cfg";
- const std::string npath = DrBaseDirectory + PSS + "mednafen.cfg";
+ const std::string opath09x = DrBaseDirectory + MDFN_PSS + "mednafen-09x.cfg";
+ const std::string npath = DrBaseDirectory + MDFN_PSS + "mednafen.cfg";
  bool mednafencfg_old = false;	// "mednafen.cfg" is old(0.8.x or earlier), or nonexistent
 
  try
@@ -1888,16 +1941,16 @@ static bool LoadSettings(void)
     {
      assert(s.desc.type == MDFNST_STRING);
      //
-     const size_t s_value_len = strlen(s.value);
+     const size_t s_value_len = strlen(s.value[0]);
      if(s_value_len > 0)
      {
       int req;
 
-      if((req = MultiByteToWideChar(CP_ACP, 0, s.value, s_value_len, NULL, 0)) > 0) // not CP_THREAD_ACP
+      if((req = MultiByteToWideChar(CP_ACP, 0, s.value[0], s_value_len, NULL, 0)) > 0) // not CP_THREAD_ACP
       {
        std::unique_ptr<char16_t[]> ws(new char16_t[req]);
 
-       if(MultiByteToWideChar(CP_ACP, 0, s.value, s_value_len, (wchar_t*)ws.get(), req) == req)
+       if(MultiByteToWideChar(CP_ACP, 0, s.value[0], s_value_len, (wchar_t*)ws.get(), req) == req)
         MDFNI_SetSetting(s.desc.name, UTF16_to_UTF8(ws.get(), req));
        else
         throw MDFN_Error(0, _("Error converting value of setting \"%s\" to UTF-8."), s.desc.name);
@@ -1923,9 +1976,155 @@ static bool LoadSettings(void)
 
 static void SaveSettings(void)
 {
- const std::string npath = DrBaseDirectory + PSS + "mednafen.cfg";
+ try
+ {
+  const std::string npath = DrBaseDirectory + MDFN_PSS + "mednafen.cfg";
 
- MDFNI_SaveSettings(npath.c_str());
+  MDFNI_SaveSettings(npath.c_str());
+ }
+ catch(std::exception& e)
+ {
+  MDFND_OutputNotice(MDFN_NOTICE_ERROR, e.what());
+ }
+}
+
+static bool MainVideoLoop(void)
+{
+ bool FirstVideoSync = true;
+ bool VideoError = false;
+
+ while(MDFN_LIKELY(!NeedExitNow))
+ {
+  MThreading::Mutex_Lock(VTMutex);	/* Lock mutex */
+
+  try
+  {
+   PumpWrap();
+
+   if(MDFN_UNLIKELY(NeedVideoSync))
+   {
+    VideoError = false;     // Set to false before calling Video_Sync().
+    //
+    Video_Sync(CurGame);
+    PumpWrap();
+    //
+    FirstVideoSync = false; // Set to false AFTER calling Video_Sync().
+    NeedVideoSync = false;  // same
+   }
+
+   if(NeededWMInputBehavior_Dirty)
+   {
+    Video_SetWMInputBehavior(NeededWMInputBehavior);
+    NeededWMInputBehavior_Dirty = false;
+   }
+
+   {
+    const int vtr = VTReady.load(std::memory_order_acquire);
+
+    if(vtr >= 0)
+    {
+     if(!VideoError)
+      BlitScreen(SoftFB[vtr].surface.get(), &SoftFB[vtr].rect, SoftFB[vtr].lw.get(), VTRotated, SoftFB[vtr].field, VTSSnapshot);
+
+     // Set to -1 after we're done blitting everything(including on-screen display stuff), and NOT just the emulated system's video surface.
+     VTReady.store(-1, std::memory_order_release);
+    }
+   }
+  }
+  catch(std::exception& e)
+  {
+   MDFND_OutputNotice(MDFN_NOTICE_ERROR, e.what());
+
+   if(FirstVideoSync || VideoError)
+   {
+    MThreading::Mutex_Unlock(VTMutex);   /* Unlock mutex */
+    return false;
+   }
+
+   VideoError = true;
+   NeedVideoSync = false;
+  }
+
+  MThreading::Mutex_Unlock(VTMutex);   /* Unlock mutex */
+
+  MThreading::Sem_TimedWait(VTWakeupSem, 1);
+ }
+
+ return true;
+}
+
+static bool LoadAndRunGame(const char* force_module, const char* path)
+{
+ bool ret = true;
+
+ VTReady.store(-1, std::memory_order_release);
+
+ NeedExitNow = false;
+
+ try
+ {
+  if(LoadGame(force_module, path))
+  {
+   const uint64 vt_affinity = MDFN_GetSettingUI("affinity.video");
+   const uint64 gt_affinity = MDFN_GetSettingUI("affinity.emu");
+
+   for(int i = 0; i < 2; i++)
+   {
+    SoftFB[i].surface.reset(new MDFN_Surface(NULL, CurGame->fb_width, CurGame->fb_height, CurGame->fb_width, MDFN_PixelFormat::ABGR32_8888));
+    SoftFB[i].lw.reset(new int32[CurGame->fb_height]);
+    memset(SoftFB[i].lw.get(), 0, sizeof(int32) * CurGame->fb_height);
+
+    SoftFB[i].surface->Fill(0, 0, 0, 0);
+
+    //
+    // Debugger step mode, cheat interface, and frame advance mode rely on the previous backbuffer being valid in certain situations.  Initialize some stuff here so that
+    // reliance will still work even immediately after startup.
+    SoftFB[i].rect.w = std::min<int32>(16, SoftFB[i].surface->w);
+    SoftFB[i].rect.h = std::min<int32>(16, SoftFB[i].surface->h);
+    SoftFB[i].lw[0] = ~0;
+   }
+
+   FPS_Init(MDFN_GetSettingUI("fps.position"), MDFN_GetSettingUI("fps.scale"), MDFN_GetSettingUI("fps.font"), MDFN_GetSettingUI("fps.textcolor"), MDFN_GetSettingUI("fps.bgcolor"));
+   if(MDFN_GetSettingB("fps.autoenable"))
+    FPS_ToggleView();
+   //
+   //
+   NeedVideoSync = true;	// Set to true before creating game thread.
+   GameThreadRun = true;
+   GameThread = MThreading::Thread_Create(GameLoop, NULL, "MDFN Emulation");
+
+   if(gt_affinity)
+    MThreading::Thread_SetAffinity(GameThread, gt_affinity);
+
+   if(vt_affinity)
+    MThreading::Thread_SetAffinity(NULL, vt_affinity);
+   //
+   //
+   //
+   if(!MainVideoLoop())
+    ret = false;
+  }
+  else
+   ret = false;
+ }
+ catch(std::exception& e)
+ {
+  MDFND_OutputNotice(MDFN_NOTICE_ERROR, e.what());
+  ret = false;
+ }
+ //
+ //
+ CloseGame();
+
+ FPS_Kill();
+
+ for(int i = 0; i < 2; i++)
+ {
+  SoftFB[i].surface.reset(nullptr);
+  SoftFB[i].lw.reset(nullptr);
+ }
+
+ return ret;
 }
 
 #ifdef WIN32
@@ -1975,7 +2174,6 @@ extern "C"
  void __set_app_type(int);
  extern int mingw_app_type;
 }
-
 __attribute__((force_align_arg_pointer))	// Not sure what's going on to cause this to be needed.
 #endif
 int main(int argc, char *argv[])
@@ -2007,7 +2205,6 @@ int main(int argc, char *argv[])
 	//
 	//
 	std::unique_ptr<FileStream> lockfs;
-	int FatalVideoError = -1;
 
 	#ifdef WIN32
 	if(!MSW_GetArgcArgv(&argc, &argv))
@@ -2020,9 +2217,13 @@ int main(int argc, char *argv[])
 	}
 	#endif
 	// Place before calls to SDL_Init()
-	putenv(strdup("SDL_DISABLE_LOCK_KEYS=1"));
 	SDL_SetHint(SDL_HINT_NO_SIGNAL_HANDLERS, "1");
-	SDL_SetHint(SDL_HINT_GRAB_KEYBOARD, "1");
+	#ifdef SDL_HINT_ALLOW_ALT_TAB_WHILE_GRABBED
+	SDL_SetHint(SDL_HINT_ALLOW_ALT_TAB_WHILE_GRABBED, "0");
+	#endif
+	#ifdef SDL_HINT_APP_NAME
+	SDL_SetHint(SDL_HINT_APP_NAME, "Mednafen");
+	#endif
 	//
 	//
 	//
@@ -2045,8 +2246,6 @@ int main(int argc, char *argv[])
 
         //
 
-	MDFNDHaveFocus = false;
-
 	DrBaseDirectory = GetBaseDirectory();
 
 	#ifdef ENABLE_NLS
@@ -2093,14 +2292,17 @@ int main(int argc, char *argv[])
 	MDFN_indent(1);
 
         MDFN_printf(_("Build information:\n"));
-        MDFN_indent(2);
-        PrintCompilerVersion();
-	//PrintGLIBCXXInfo();
-        PrintZLIBVersion();
-	PrintLIBICONVVersion();
-        PrintSDLVersion();
-        PrintLIBFLACVersion();
-        MDFN_indent(-2);
+	{
+	 MDFN_AutoIndent aind(2);
+
+	 PrintCompilerVersion();
+	 //PrintGLIBCXXInfo();
+	 PrintZLIBVersion();
+	 PrintLIBICONVVersion();
+	 if(!CheckPrintSDLVersion())
+	  return -1;
+	 PrintLIBFLACVersion();
+	}
 
         MDFN_printf(_("Base directory: %s\n"), MDFN_strhumesc(DrBaseDirectory).c_str());
 
@@ -2150,7 +2352,7 @@ int main(int argc, char *argv[])
 	try
 	{
  	 // Create base directory
-	 NVFS.create_missing_dirs(DrBaseDirectory + PSS);
+	 NVFS.create_missing_dirs(DrBaseDirectory + MDFN_PSS);
 	}
 	catch(std::exception &e)
 	{
@@ -2165,7 +2367,7 @@ int main(int argc, char *argv[])
 	 MDFN_AutoIndent aind(1);
 	 try
 	 {
-	  lockfs.reset(new FileStream(DrBaseDirectory + PSS + "mednafen.lck", FileStream::MODE_WRITE_INPLACE, -1));
+	  lockfs.reset(new FileStream(DrBaseDirectory + MDFN_PSS + "mednafen.lck", FileStream::MODE_WRITE_INPLACE, -1));
 	 }
 	 catch(MDFN_Error& e)
 	 {
@@ -2222,192 +2424,46 @@ int main(int argc, char *argv[])
 	//
 	//
 	//
-	InstallSignalHandlers();
-	//
-	//
-	//
-
-	/* Now the fun begins! */
-	/* Run the video and event pumping in the main thread, and create a 
-	   secondary thread to run the game in(and do sound output, since we use
-	   separate sound code which should be thread safe(?)).
-	*/
 	int ret = 0;
 
-	VTMutex = MThreading::Mutex_Create();
-        EVMutex = MThreading::Mutex_Create();
-
-	VTWakeupSem = MThreading::Sem_Create();
-	//
-	Video_Init();
-	//
-	JoystickManager::Init();
-	JoystickManager::SetAnalogThreshold(MDFN_GetSettingF("analogthreshold") / 100);
-
-#if 0
-for(int zgi = 1; zgi < argc; zgi++)// start game load test loop
-{
- needie = argv[zgi];
-#endif
-
-	VTReady.store(-1, std::memory_order_release);
-
-	NeedExitNow = 0;
-
-	#if 0
+	try
 	{
-	 int64 start_ticks = Time::MonoUS();
-
-	 for(int i = 0; i < 65536; i++)
-	  MDFN_GetSettingB("gg.forcemono");
-
-	 printf("%lld\n", (long long)(Time::MonoUS() - start_ticks));
-	}
-	#endif
-
- try
- {
-        if(LoadGame(force_module_arg, needie))
-        {
-         NeedVideoSync = 1;	// Set to 1 before creating game thread.
-	 //
-	 //
-	 const uint64 vt_affinity = MDFN_GetSettingUI("affinity.video");
-         const uint64 gt_affinity = MDFN_GetSettingUI("affinity.emu");
-
-	 GameThreadRun = 1;
-	 GameThread = MThreading::Thread_Create(GameLoop, NULL, "MDFN Emulation");
-
-	 if(gt_affinity)
-	  MThreading::Thread_SetAffinity(GameThread, gt_affinity);
-
-	 if(vt_affinity)
-	  MThreading::Thread_SetAffinity(NULL, vt_affinity);
+	 InstallSignalHandlers();
 	 //
 	 //
 	 //
-	 uint32 pitch32 = CurGame->fb_width; 
-	 MDFN_PixelFormat nf(MDFN_PixelFormat::ABGR32_8888);
 
-         for(int i = 0; i < 2; i++)
+	 /* Now the fun begins! */
+	 /* Run the video and event pumping in the main thread, and create a 
+	    secondary thread to run the game in(and do sound output, since we use
+	    separate sound code which should be thread safe(?)).
+	 */
+	 VTMutex = MThreading::Mutex_Create();
+         EVMutex = MThreading::Mutex_Create();
+
+	 VTWakeupSem = MThreading::Sem_Create();
+	 //
+	 Video_Init();
+	 //
+	 JoystickManager::Init();
+	 JoystickManager::SetAnalogThreshold(MDFN_GetSettingF("analogthreshold") / 100);
+
+	 #if 0
+	 for(int zgi = 1; zgi < argc; zgi++)// start game load test loop
 	 {
-	  SoftFB[i].surface.reset(new MDFN_Surface(NULL, CurGame->fb_width, CurGame->fb_height, pitch32, nf));
-	  SoftFB[i].lw.reset(new int32[CurGame->fb_height]);
-	  memset(SoftFB[i].lw.get(), 0, sizeof(int32) * CurGame->fb_height);
-
-	  SoftFB[i].surface->Fill(0, 0, 0, 0);
-
-	  //
-	  // Debugger step mode, cheat interface, and frame advance mode rely on the previous backbuffer being valid in certain situations.  Initialize some stuff here so that
-	  // reliance will still work even immediately after startup.
-	  SoftFB[i].rect.w = std::min<int32>(16, SoftFB[i].surface->w);
-	  SoftFB[i].rect.h = std::min<int32>(16, SoftFB[i].surface->h);
-	  SoftFB[i].lw[0] = ~0;
-	 }
-
-         FPS_Init(MDFN_GetSettingUI("fps.position"), MDFN_GetSettingUI("fps.scale"), MDFN_GetSettingUI("fps.font"), MDFN_GetSettingUI("fps.textcolor"), MDFN_GetSettingUI("fps.bgcolor"));
-	 if(MDFN_GetSettingB("fps.autoenable"))
-          FPS_ToggleView();
-        }
-	else
-	{
-	 ret = -1;
-	 NeedExitNow = 1;
-	}
- }
- catch(std::exception& e)
- {
-  MDFND_OutputNotice(MDFN_NOTICE_ERROR, e.what());
-  ret = -1;
-  NeedExitNow = 1;
- }
-
-	while(MDFN_LIKELY(!NeedExitNow))
-	{
-	 MThreading::Mutex_Lock(VTMutex);	/* Lock mutex */
-
-	 try
-	 {
-	  if(FatalVideoError > 0)
-	  {
-	   PumpWrap();
-	   NeedVideoSync = 0;
-	   NeededWMInputBehavior_Dirty = false;
-	   VTReady.store(-1, std::memory_order_release);
-	  }
-	  else
-	  {
-	   PumpWrap();
-
-	   if(MDFN_UNLIKELY(NeedVideoSync))
-           {
-	    Video_Sync(CurGame);
-	    PumpWrap();
-	    //
-	    NeedVideoSync = 0;
-           }
-
-	   if(NeededWMInputBehavior_Dirty)
-	   {
-	    Video_SetWMInputBehavior(NeededWMInputBehavior);
-	    NeededWMInputBehavior_Dirty = false;
-	   }
-
-	   {
-	    const int vtr = VTReady.load(std::memory_order_acquire);
-
-            if(vtr >= 0)
-            {
-             BlitScreen(SoftFB[vtr].surface.get(), &SoftFB[vtr].rect, SoftFB[vtr].lw.get(), VTRotated, SoftFB[vtr].field, VTSSnapshot);
-
-	     // Set to -1 after we're done blitting everything(including on-screen display stuff), and NOT just the emulated system's video surface.
-             VTReady.store(-1, std::memory_order_release);
-            }
-	   }
-	  }
-	  //
-	  //
-	  //
-	  if(FatalVideoError < 0)
-	   FatalVideoError = 0;
-	 }
-	 catch(std::exception& e)
-	 {
-	  MDFND_OutputNotice(MDFN_NOTICE_ERROR, e.what());
-	  if(FatalVideoError == 0)
-	  {
- 	   MThreading::Mutex_Unlock(VTMutex);   /* Unlock mutex */ 
-	   FatalVideoError = 1;
-	  }
-	  else
-	  {	  
+	  needie = argv[zgi];
+	 #endif
+	  if(!LoadAndRunGame(force_module_arg, needie))
 	   ret = -1;
-           NeedExitNow = 1;
-	   MThreading::Mutex_Unlock(VTMutex);   /* Unlock mutex */
-	   goto VideoErrorExit;
-	  }
+	 #if 0
 	 }
-
-         MThreading::Mutex_Unlock(VTMutex);   /* Unlock mutex */
-
-	 MThreading::Sem_TimedWait(VTWakeupSem, 1);
+	 #endif
 	}
-	VideoErrorExit:;
-	//
-	//
-	//
-	CloseGame();
-
-	FPS_Kill();
-
-	for(int i = 0; i < 2; i++)
+	catch(std::exception& e)
 	{
-	 SoftFB[i].surface.reset(nullptr);
-	 SoftFB[i].lw.reset(nullptr);
+	 MDFND_OutputNotice(MDFN_NOTICE_ERROR, e.what());
+	 ret = -1;
 	}
-#if 0
-} // end game load test loop
-#endif
 
 	MThreading::Sem_Destroy(VTWakeupSem);
 
