@@ -1,12 +1,13 @@
 #ifndef __MDFN_DRIVERS_SOUND_H
 #define __MDFN_DRIVERS_SOUND_H
 
+bool Sound_Init(MDFNGI *gi) MDFN_COLD;
+void Sound_Kill(void) noexcept MDFN_COLD;
+
 bool Sound_NeedReInit(void);
 
-bool Sound_Init(MDFNGI *gi);
 void Sound_Write(int16 *Buffer, int Count);
 void Sound_WriteSilence(int ms);
-bool Sound_Kill(void);
 
 uint32 Sound_CanWrite(void);
 
