@@ -72,7 +72,7 @@ class SDL_to_MDFN_Surface_Wrapper : public MDFN_Surface
   pixels16 = NULL;
   pixels = NULL;
 
-  if(ss->format->BitsPerPixel == 16)
+  if(ss->format->BytesPerPixel == 2)
   {
    pixels16 = (uint16*)ss->pixels;
    pitchinpix = ss->pitch >> 1;
@@ -191,9 +191,7 @@ static const MDFNSetting GlobalVideoSettings[] =
 
  { "video.force_bbclear", MDFNSF_NOFLAGS, gettext_noop("Force backbuffer clear before drawing."), gettext_noop("Enabling may result in a noticeable negative impact on performance with the \"softfb\" video driver, and with the \"opengl\" video driver on underpowered GPUs."), MDFNST_BOOL, "0" },
 
- { "video.glvsync", MDFNSF_NOFLAGS, gettext_noop("Attempt to synchronize OpenGL page flips to vertical retrace period."), 
-			       gettext_noop("Note: Additionally, if the environment variable \"__GL_SYNC_TO_VBLANK\" does not exist, then it will be created and set to the value specified for this setting.  This has the effect of forcibly enabling or disabling vblank synchronization when running under Linux with NVidia's drivers."),
-				MDFNST_BOOL, "1" },
+ { "video.glvsync", MDFNSF_NOFLAGS, gettext_noop("Attempt to synchronize OpenGL page flips to vertical retrace period."), NULL, MDFNST_BOOL, "1" },
 
  { "video.disable_composition", MDFNSF_NOFLAGS, gettext_noop("Attempt to disable desktop composition."), gettext_noop("Currently, this setting only has an effect on Windows Vista and Windows 7(and probably the equivalent server versions as well)."), MDFNST_BOOL, "1" },
 
@@ -302,10 +300,10 @@ void Video_MakeSettings(void)
 
  static const char *CSD_xscalefs = gettext_noop("Scaling factor for the X axis in fullscreen mode.");
  static const char *CSD_yscalefs = gettext_noop("Scaling factor for the Y axis in fullscreen mode.");
- static const char *CSDE_xyscalefs = gettext_noop("For this setting to have any effect, the \"<system>.stretch\" setting must be set to \"0\".");
+ static const char *CSDE_xyscalefs = gettext_noop("For this setting to have any effect, the \"\5<system>.stretch\" setting must be set to \"0\".");
 
  static const char *CSD_scanlines = gettext_noop("Enable scanlines with specified opacity.");
- static const char *CSDE_scanlines = gettext_noop("Opacity is specified in %; IE a value of \"100\" will give entirely black scanlines.\n\nNegative values are the same as positive values for non-interlaced video, but for interlaced video will cause the scanlines to be overlaid over the previous(if the video.deinterlacer setting is set to \"weave\", the default) field's lines.");
+ static const char *CSDE_scanlines = gettext_noop("Opacity is specified in %; IE a value of \"100\" will give entirely black scanlines.\n\nNegative values are the same as positive values for non-interlaced video, but for interlaced video will cause the scanlines to be overlaid over the previous(if the \"\5video.deinterlacer\" setting is set to \"weave\", the default) field's lines.");
 
  static const char *CSD_stretch = gettext_noop("Stretch to fill screen.");
  static const char *CSDvideo_settingsip = gettext_noop("Enable (bi)linear interpolation.");
@@ -314,7 +312,7 @@ void Video_MakeSettings(void)
  static const char *CSDE_special = gettext_noop("The destination rectangle is NOT altered by this setting, so if you have xscale and yscale set to \"2\", and try to use a 3x scaling filter like hq3x, the image is not going to look that great. The nearest-neighbor scalers are intended for use with bilinear interpolation enabled, for a sharper image, though the \"autoipsharper\" shader may provide better results.");
 
  static const char *CSD_shader = gettext_noop("Enable specified OpenGL shader.");
- static const char *CSDE_shader = gettext_noop("Obviously, this will only work with the OpenGL \"video.driver\" setting, and only on cards and OpenGL implementations that support shaders, otherwise you will get a black screen, or Mednafen may display an error message when starting up. When a shader is enabled, the \"<system>.videoip\" setting is ignored.");
+ static const char *CSDE_shader = gettext_noop("Obviously, this will only work with the OpenGL \"\5video.driver\" setting, and only on cards and OpenGL implementations that support shaders, otherwise you will get a black screen, or Mednafen may display an error message when starting up. When a shader is enabled, the \"\5<system>.videoip\" setting is ignored.");
 
  for(unsigned int i = 0; i < MDFNSystems.size() + 1; i++)
  {
@@ -382,7 +380,7 @@ void Video_MakeSettings(void)
   AddSystemSetting(sysname, "shader.goat.vdiv", gettext_noop("Constant RGB vertical divergence."), nullptr, MDFNST_FLOAT, "0.50", "-2.00", "2.00");
   AddSystemSetting(sysname, "shader.goat.pat", gettext_noop("Mask pattern."), nullptr, MDFNST_ENUM, "goatron", NULL, NULL, NULL, NULL, GoatPat_List);
   AddSystemSetting(sysname, "shader.goat.tp", gettext_noop("Transparency of otherwise-opaque mask areas."), nullptr, MDFNST_FLOAT, "0.50", "0.00", "1.00");
-  AddSystemSetting(sysname, "shader.goat.fprog", gettext_noop("Force interlaced video to be treated as progressive."), gettext_noop("When disabled, the default, the \"video.deinterlacer\" setting is effectively ignored with respect to what appears on the screen, unless it's set to \"blend\" or \"blend_rg\".  When enabled, it may be prudent to disable the scanlines effect controlled by the *.goat.slen setting, or else the scanline effect may look objectionable."), MDFNST_BOOL, "0");
+  AddSystemSetting(sysname, "shader.goat.fprog", gettext_noop("Force interlaced video to be treated as progressive."), gettext_noop("When disabled, the default, the \"\5video.deinterlacer\" setting is effectively ignored with respect to what appears on the screen, unless it's set to \"blend\" or \"blend_rg\".  When enabled, it may be prudent to disable the scanlines effect controlled by the \"\5<system>.shader.goat.slen\" setting, or else the scanline effect may look objectionable."), MDFNST_BOOL, "0");
   AddSystemSetting(sysname, "shader.goat.slen", gettext_noop("Enable scanlines effect."), nullptr, MDFNST_BOOL, "1");
  }
 
@@ -475,6 +473,8 @@ static MDFN_PixelFormat game_pf; // Pixel format for game texture/surface
 static MDFN_PixelFormat osd_pf;	 // Pixel format for OSD textures/surfaces
 static MDFN_PixelFormat emu_pf;
 
+static void GrabbyGoAway(void);
+
 static INLINE void MarkNeedBBClear(void)
 {
  NeedClear = 15;
@@ -519,9 +519,7 @@ void Video_Kill(void)
   if(SDL_GetWindowFlags(window) & SDL_WINDOW_FULLSCREEN)
    SDL_SetWindowFullscreen(window, 0);
   //
-  SDL_SetRelativeMouseMode(SDL_FALSE);
-  SDL_ShowCursor(SDL_TRUE);
-  SDL_SetWindowGrab(window, SDL_FALSE);
+  GrabbyGoAway();
 
   SDL_DestroyWindow(window);
   window = nullptr;
@@ -550,9 +548,7 @@ bool Video_ErrorPopup(bool warning, const char* title, const char* text)
  if(window)
  {
   // Best not to drive the user stark raving mad.
-  SDL_SetRelativeMouseMode(SDL_FALSE);
-  SDL_ShowCursor(SDL_TRUE);
-  SDL_SetWindowGrab(window, SDL_FALSE);
+  GrabbyGoAway();
   // should we or shouldn't we...: SDL_SetHint(SDL_HINT_WINDOWS_NO_CLOSE_ON_ALT_F4, "0");
  }
 #if 0
@@ -701,21 +697,65 @@ static bool GenerateFullscreenDestRect(void)
  return screen_dest_rect.w < 16384 && screen_dest_rect.h < 16384;
 }
 
-static bool weset_glstvb = false; 
+// Called on error popups and exit.
+static void GrabbyGoAway(void)
+{
+ SDL_SetRelativeMouseMode(SDL_FALSE);
+ SDL_ShowCursor(SDL_TRUE);
+
+#if SDL_VERSION_ATLEAST(2, 0, 16)
+ SDL_SetWindowKeyboardGrab(window, SDL_FALSE);
+ SDL_SetWindowMouseGrab(window, SDL_FALSE);
+#else
+ SDL_SetWindowGrab(window, SDL_FALSE);
+#endif
+}
 
 void Video_SetWMInputBehavior(const WMInputBehavior& beeeeees)
 {
  CurWMIB = beeeeees;
  //
  const bool fs = (SDL_GetWindowFlags(window) & SDL_WINDOW_FULLSCREEN);
- const bool grab = CurWMIB.Grab;
- const bool relm = CurWMIB.MouseRel && !CurWMIB.MouseAbs && !CurWMIB.Cursor && (grab || fs);
+ const bool grab_keyboard = CurWMIB.Grab_Keyboard;
+ const bool grab_mouse = CurWMIB.Grab_Mouse;
+ const bool relm = CurWMIB.MouseRel && !CurWMIB.MouseAbs && !CurWMIB.Cursor && (grab_mouse || fs);
  const bool curse = !relm && (CurWMIB.Cursor || video_settings.cursorvis == CURSORVIS_VISIBLE);
 
- //printf("Grab: %d, RelM: %d, Curse: %d\n", grab, relm, curse);
-
- if(grab)
+#if SDL_VERSION_ATLEAST(2, 0, 16)
+ if(grab_keyboard || grab_mouse)
  {
+  SDL_ShowWindow(window);
+  SDL_RaiseWindow(window);
+
+  if(grab_keyboard)
+   SDL_SetWindowKeyboardGrab(window, SDL_TRUE);
+
+  if(grab_mouse)
+   SDL_SetWindowMouseGrab(window, SDL_TRUE);
+ }
+
+ SDL_ShowCursor(SDL_FALSE);
+ SDL_SetRelativeMouseMode(relm ? SDL_TRUE : SDL_FALSE);
+ SDL_ShowCursor(curse ? SDL_TRUE : SDL_FALSE);
+
+ if(!grab_keyboard)
+  SDL_SetWindowKeyboardGrab(window, SDL_FALSE);
+
+ if(!grab_mouse)
+  SDL_SetWindowMouseGrab(window, SDL_FALSE);
+#else
+ const bool any_grab = (grab_keyboard || grab_mouse);
+
+ if(any_grab && (bool)(SDL_GetWindowFlags(window) & SDL_WINDOW_INPUT_GRABBED))
+ {
+  SDL_SetWindowGrab(window, SDL_FALSE);
+  SDL_SetRelativeMouseMode(SDL_FALSE);
+ }
+
+ if(any_grab)
+ {
+  SDL_SetHint(SDL_HINT_GRAB_KEYBOARD, grab_keyboard ? "1" : "0");
+
   SDL_ShowWindow(window);
   SDL_RaiseWindow(window);
   SDL_SetWindowGrab(window, SDL_TRUE);
@@ -725,10 +765,11 @@ void Video_SetWMInputBehavior(const WMInputBehavior& beeeeees)
  SDL_SetRelativeMouseMode(relm ? SDL_TRUE : SDL_FALSE);
  SDL_ShowCursor(curse ? SDL_TRUE : SDL_FALSE);
 
- if(!grab)
+ if(!any_grab)
   SDL_SetWindowGrab(window, SDL_FALSE);
+#endif
 
- SDL_SetHint(SDL_HINT_WINDOWS_NO_CLOSE_ON_ALT_F4, grab ? "1" : "0");
+ SDL_SetHint(SDL_HINT_WINDOWS_NO_CLOSE_ON_ALT_F4, grab_keyboard ? "1" : "0");
 }
 
 #if 0
@@ -794,29 +835,6 @@ void Video_Sync(MDFNGI *gi)
   }
  }
  #endif
-
- if(!getenv("__GL_SYNC_TO_VBLANK") || weset_glstvb)
- {
-  if(MDFN_GetSettingB("video.glvsync"))
-  {
-   #if HAVE_PUTENV
-   static char gl_pe_string[] = "__GL_SYNC_TO_VBLANK=1";
-   putenv(gl_pe_string); 
-   #elif HAVE_SETENV
-   setenv("__GL_SYNC_TO_VBLANK", "1", 1);
-   #endif
-  }
-  else
-  {
-   #if HAVE_PUTENV
-   static char gl_pe_string[] = "__GL_SYNC_TO_VBLANK=0";
-   putenv(gl_pe_string); 
-   #elif HAVE_SETENV
-   setenv("__GL_SYNC_TO_VBLANK", "0", 1);
-   #endif
-  }
-  weset_glstvb = true;
- }
  //
  osd_alpha_blend = MDFN_GetSettingB("osd.alpha_blend");
 
@@ -1095,6 +1113,16 @@ void Video_Sync(MDFNGI *gi)
   if(yres > 0)
    trymode.h = yres;
 
+  if(vdriver != VDRIVER_OPENGL)
+  {
+   if(!(trymode.format == SDL_PIXELFORMAT_RGB565 && (gi->ExtraVideoFormatSupport & EVFSUPPORT_RGB565)) &&
+      !(trymode.format == SDL_PIXELFORMAT_RGB555 && (gi->ExtraVideoFormatSupport & EVFSUPPORT_RGB555)) &&
+      !(SDL_BYTESPERPIXEL(trymode.format) == 4 && trymode.format != SDL_PIXELFORMAT_ARGB2101010))
+   {
+    trymode.format = SDL_PIXELFORMAT_RGB888;
+   }
+  }
+
   if(!SDL_GetClosestDisplayMode(dindex, &trymode, &mode))
   {
    MDFN_Notify(MDFN_NOTICE_WARNING, _("Reverting to windowed mode because no modes big enough for %dx%d."), trymode.w, trymode.h);
@@ -1164,18 +1192,33 @@ void Video_Sync(MDFNGI *gi)
    if(!(screen = SDL_GetWindowSurface(window)))
     throw MDFN_Error(0, _("SDL_GetWindowSurface() failed: %s"), SDL_GetError());
 
-   if(screen->format->BitsPerPixel != 32)
-    throw MDFN_Error(0, _("Window surface bit depth(%ubpp) is not supported by Mednafen."), screen->format->BitsPerPixel);
+   if(screen->format->BytesPerPixel == 4 &&
+	screen->format->Rloss == 0 && screen->format->Gloss == 0 && screen->format->Bloss == 0 &&
+	!((screen->format->Rshift | screen->format->Gshift | screen->format->Bshift) & 0x7))
+   {
+    int rs = screen->format->Rshift;
+    int gs = screen->format->Gshift;
+    int bs = screen->format->Bshift;
 
-   int rs = screen->format->Rshift;
-   int gs = screen->format->Gshift;
-   int bs = screen->format->Bshift;
+    int as = 0;
+    while(as == rs || as == gs || as == bs) // Find unused 8-bits to use as our alpha channel
+     as += 8;
 
-   int as = 0;
-   while(as == rs || as == gs || as == bs) // Find unused 8-bits to use as our alpha channel
-    as += 8;
+    osd_pf = game_pf = emu_pf = MDFN_PixelFormat(MDFN_COLORSPACE_RGB, 4, rs, gs, bs, as, 8, 8, 8, 8);
+   }
+   else if(screen->format->format == SDL_PIXELFORMAT_RGB565 && (gi->ExtraVideoFormatSupport & EVFSUPPORT_RGB565))
+    osd_pf = game_pf = emu_pf = MDFN_PixelFormat::RGB16_565;
+   else if(screen->format->format == SDL_PIXELFORMAT_RGB555 && (gi->ExtraVideoFormatSupport & EVFSUPPORT_RGB555))
+    osd_pf = game_pf = emu_pf = MDFN_PixelFormat::IRGB16_1555;
+   else
+   {
+    const char* pfname = SDL_GetPixelFormatName(screen->format->format);
 
-   osd_pf = game_pf = emu_pf = MDFN_PixelFormat(MDFN_COLORSPACE_RGB, 4, rs, gs, bs, as, 8, 8, 8, 8);
+    if(screen->format->format == SDL_PIXELFORMAT_RGB565 || screen->format->format == SDL_PIXELFORMAT_RGB555)
+     throw MDFN_Error(0, _("Window surface pixel format \"%s\" is not supported by module \"%s\"."), pfname, gi->shortname);
+    else
+     throw MDFN_Error(0, _("Window surface pixel format \"%s\" is not supported by Mednafen."), pfname);
+   }
   }
  }
 
@@ -1299,7 +1342,8 @@ void Video_Init(void)
  CurWMIB.Cursor = true;
  CurWMIB.MouseAbs = true;
  CurWMIB.MouseRel = false;
- CurWMIB.Grab = false;
+ CurWMIB.Grab_Keyboard = false;
+ CurWMIB.Grab_Mouse = false;
  //
  for(unsigned i = 0; i < 2 && !window; i++)
  {
