@@ -5,6 +5,9 @@
 #include <mednafen/mednafen.h>
 #include <mednafen/lepacker.h>
 
+namespace Mednafen
+{
+
 struct Ym2612_Impl;
 
 class Ym2612_Emu  {
@@ -33,7 +36,8 @@ public:
 	enum { out_chan_count = 2 }; // stereo
 	void run( sample_t* out );
 
-	void serialize(MDFN::LEPacker &slizer, bool load);
+	void serialize(Mednafen::LEPacker &slizer, bool load);
 };
 
+}
 #endif
