@@ -32,7 +32,7 @@
 
 #include "huc.h"
 
-namespace PCE_Fast
+namespace MDFN_IEN_PCE_FAST
 {
 
 static const uint8 BRAM_Init_String[8] = { 'H', 'U', 'B', 'M', 0x00, 0x88, 0x10, 0x80 }; //"HUBM\x00\x88\x10\x80";
@@ -130,7 +130,7 @@ static void LoadSaveMemory(const std::string& path, uint8* const data, const uin
   const uint64 fp_size_tmp = fp.size();
 
   if(fp_size_tmp != len)
-   throw MDFN_Error(0, _("Save game memory file \"%s\" is an incorrect size(%llu bytes).  The correct size is %llu bytes."), path.c_str(), (unsigned long long)fp_size_tmp, (unsigned long long)len);
+   throw MDFN_Error(0, _("Save game memory file \"%s\" is an incorrect size(%llu bytes).  The correct size is %llu bytes."), MDFN_strhumesc(path).c_str(), (unsigned long long)fp_size_tmp, (unsigned long long)len);
 
   fp.read(data, len);
  }
@@ -291,7 +291,7 @@ void HuC_LoadCD(const std::string& bios_path)
 
  try
  {
-  MDFNFILE fp(&NVFS, bios_path.c_str(), KnownBIOSExtensions, _("CD BIOS"));
+  MDFNFILE fp(&NVFS, bios_path, KnownBIOSExtensions, _("CD BIOS"));
 
   memset(ROMSpace, 0xFF, 262144);
 
