@@ -447,9 +447,8 @@ static MDFN_COLD bool TestMagicCD(std::vector<CDInterface*> *CDInterfaces)
  return(ret);
 }
 
-static MDFN_COLD bool DetectSGXCD(std::vector<CDInterface*>* CDInterfaces)
+static MDFN_COLD bool DetectSGXCD(CDInterface* cdiface)
 {
- CDInterface* cdiface = (*CDInterfaces)[0];
  CDUtility::TOC toc;
  uint8 sector_buffer[2048];
  bool ret = false;
@@ -481,7 +480,7 @@ static MDFN_COLD void LoadCD(std::vector<CDInterface*> *CDInterfaces)
   std::string bios_path = MDFN_MakeFName(MDFNMKF_FIRMWARE, 0, MDFN_GetSettingS("pce_fast.cdbios"));
 
   IsHES = 0;
-  IsSGX = DetectSGXCD(CDInterfaces);
+  IsSGX = CDInterfaces->size() && DetectSGXCD((*CDInterfaces)[0]);
 
   LoadCommonPre();
 
