@@ -19,7 +19,9 @@
 ** 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
-#pragma GCC optimize ("unroll-loops")
+#if defined(__GNUC__) && !defined(__clang__)
+ #pragma GCC optimize ("unroll-loops")
+#endif
 
 #include "psx.h"
 #include "timer.h"
@@ -702,7 +704,7 @@ static void ProcessFIFO(void)
 	break;
 
   case PS_GPU::INCMD_FBREAD:
-	PSX_WARNING("[GPU] Command FIFO not empty while in FB Read?!");
+	PSX_DBG(PSX_DBG_WARNING | PSX_DBG_GPU, "[GPU] Command FIFO not empty while in FB Read?!\n");
 	return;
 
   case PS_GPU::INCMD_FBWRITE:
@@ -823,7 +825,7 @@ static void ProcessFIFO(void)
   if(!command->func[abr][TexMode])
   {
    if(CB[0])
-    PSX_WARNING("[GPU] Unknown command: %08x, %d", CB[0], scanline);
+    PSX_DBG(PSX_DBG_WARNING | PSX_DBG_GPU, "[GPU] Unknown command: %08x, %d\n", CB[0], scanline);
   }
   else
   {
@@ -836,7 +838,7 @@ static void WriteCB(uint32 InData)
 {
  if(BlitterFIFO.CanRead() >= 0x10 && (InCmd != PS_GPU::INCMD_NONE || (BlitterFIFO.CanRead() - 0x10) >= Commands[BlitterFIFO.Peek() >> 24].fifo_fb_len))
  {
-  PSX_DBG(PSX_DBG_WARNING, "GPU FIFO overflow!!!\n");
+  PSX_DBG(PSX_DBG_WARNING | PSX_DBG_GPU, "[GPU] GPU FIFO overflow!!!\n");
   return;
  }
 
@@ -861,7 +863,7 @@ MDFN_FASTCALL void GPU_Write(const pscpu_timestamp_t timestamp, uint32 A, uint32
    /*
     0x40-0xFF do NOT appear to be mirrors, at least not on my PS1's GPU.
    */
-   default: PSX_WARNING("[GPU] Unknown control command %02x - %06x", command, V);
+   default: PSX_DBG(PSX_DBG_WARNING | PSX_DBG_GPU, "[GPU] Unknown control command %02x - %06x\n", command, V);
 	    break;
 
    case 0x00:	// Reset GPU
@@ -1072,8 +1074,10 @@ static INLINE uint32 MDFN_NOWARN_UNUSED ShiftHelper(uint32 val, int shamt, uint3
 }
 #endif
 
-#pragma GCC push_options
-#pragma GCC optimize("no-unroll-loops,no-peel-loops,no-crossjumping")
+#if defined(__GNUC__) && !defined(__clang__)
+ #pragma GCC push_options
+ #pragma GCC optimize("no-unroll-loops,no-peel-loops,no-crossjumping")
+#endif
 static INLINE void ReorderRGB_Var(uint32 out_Rshift, uint32 out_Gshift, uint32 out_Bshift, bool bpp24, const uint16 *src, uint32 *dest, const int32 dx_start, const int32 dx_end, int32 fb_x)
 {
      if(bpp24)	// 24bpp
@@ -1115,7 +1119,9 @@ static NO_INLINE void ReorderRGB(bool bpp24, const uint16 *src, uint32 *dest, co
 {
  ReorderRGB_Var(out_Rshift, out_Gshift, out_Bshift, bpp24, src, dest, dx_start, dx_end, fb_x);
 }
-#pragma GCC pop_options
+#if defined(__GNUC__) && !defined(__clang__)
+ #pragma GCC pop_options
+#endif
 
 MDFN_FASTCALL pscpu_timestamp_t GPU_Update(const pscpu_timestamp_t sys_timestamp)
 {
