@@ -26,7 +26,7 @@ namespace MDFN_IEN_PSX
 {
 
 InputDevice *Device_Mouse_Create(void) MDFN_COLD;
-extern IDIISG Device_Mouse_IDII;
+MDFN_HIDE extern IDIISG Device_Mouse_IDII;
 
 }
 #endif
