@@ -470,9 +470,20 @@ static const RegType DBG_Regs_PPU[] =
  { (1 << 16) | PPU_GSREG_WOBJLOG, "WOBJLOG", "WOBJLOG", 1 },
  { (1 << 16) | PPU_GSREG_TM, "TM", "TM", 1 },
  { (1 << 16) | PPU_GSREG_TS, "TS", "TS", 1 },
+ { (1 << 16) | PPU_GSREG_TMW, "TMW", "TMW", 1 },
+ { (1 << 16) | PPU_GSREG_TSW, "TSW", "TSW", 1 },
  { (1 << 16) | PPU_GSREG_CGWSEL, "CGWSEL", "CGWSEL", 1 },
  { (1 << 16) | PPU_GSREG_CGADSUB, "CGADSUB", "CGADSUB", 1 },
  { 0, "------", "", 0xFFFF },
+
+ { (1 << 16) | PPU_GSREG_BG1SC, "BG1SC", "BG1SC", 1 },
+ { (1 << 16) | PPU_GSREG_BG2SC, "BG2SC", "BG2SC", 1 },
+ { (1 << 16) | PPU_GSREG_BG3SC, "BG3SC", "BG3SC", 1 },
+ { (1 << 16) | PPU_GSREG_BG4SC, "BG4SC", "BG4SC", 1 },
+ { (1 << 16) | PPU_GSREG_BG12NBA, "BG12NBA", "BG12NBA", 1 },
+ { (1 << 16) | PPU_GSREG_BG34NBA, "BG34NBA", "BG34NBA", 1 },
+ { 0, "------", "", 0xFFFF },
+
  { (1 << 16) | PPU_GSREG_BG1HOFS, "BG1HOFS", "BG1 H Scroll", 2 },
  { (1 << 16) | PPU_GSREG_BG1VOFS, "BG1VOFS", "BG1 V Scroll", 2 },
  { (1 << 16) | PPU_GSREG_BG2HOFS, "BG2HOFS", "BG2 H Scroll", 2 },
