@@ -70,9 +70,19 @@ enum
  PPU_GSREG_TM,
  PPU_GSREG_TS,
 
+ PPU_GSREG_TMW,
+ PPU_GSREG_TSW,
+
  PPU_GSREG_CGWSEL,
  PPU_GSREG_CGADSUB,
 
+ PPU_GSREG_BG1SC,
+ PPU_GSREG_BG2SC,
+ PPU_GSREG_BG3SC,
+ PPU_GSREG_BG4SC,
+ PPU_GSREG_BG12NBA,
+ PPU_GSREG_BG34NBA,
+
  PPU_GSREG_BG1HOFS,
  PPU_GSREG_BG1VOFS,
  PPU_GSREG_BG2HOFS,
