@@ -22,8 +22,6 @@
 #ifndef __MDFN_SS_CDB_H
 #define __MDFN_SS_CDB_H
 
-class CDIF;
-
 namespace MDFN_IEN_SS
 {
 
@@ -31,7 +29,7 @@ void CDB_Init(void) MDFN_COLD;
 void CDB_Kill(void) MDFN_COLD;
 void CDB_StateAction(StateMem* sm, const unsigned load, const bool data_only) MDFN_COLD;
 
-void CDB_SetDisc(bool tray_open, CDIF *cdif) MDFN_COLD;
+void CDB_SetDisc(bool tray_open, CDInterface* cdif) MDFN_COLD;
 
 void CDB_Write_DBM(uint32 offset, uint16 DB, uint16 mask) MDFN_HOT;
 uint16 CDB_Read(uint32 offset) MDFN_HOT;
@@ -48,6 +46,25 @@ void CDB_SetClockRatio(uint32 ratio);
 void CDB_ResetCD(void);
 void CDB_SetCDActive(bool active);
 
+enum
+{
+ CDB_GSREG_HIRQ = 0,
+ CDB_GSREG_HIRQ_MASK,
+
+ CDB_GSREG_CDATA0,
+ CDB_GSREG_CDATA1,
+ CDB_GSREG_CDATA2,
+ CDB_GSREG_CDATA3,
+
+ CDB_GSREG_RESULT0,
+ CDB_GSREG_RESULT1,
+ CDB_GSREG_RESULT2,
+ CDB_GSREG_RESULT3
+};
+
+uint32 CDB_GetRegister(const unsigned id, char* const special, const uint32 special_len) MDFN_COLD;
+void CDB_SetRegister(const unsigned id, const uint32 value) MDFN_COLD;
+
 }
 
 #endif
