@@ -34,7 +34,7 @@ class IODevice_Keyboard final : public IODevice
  virtual void Power(void) override MDFN_COLD;
  virtual void UpdateInput(const uint8* data, const int32 time_elapsed) override;
  virtual void UpdateOutput(uint8* data) override;
- virtual void StateAction(StateMem* sm, const unsigned load, const bool data_only, const char* sname_prefix) override;
+ virtual void StateAction(StateMem* sm, const unsigned load, const bool data_only, const char* sname_prefix) override MDFN_COLD;
 
  virtual uint8 UpdateBus(const sscpu_timestamp_t timestamp, const uint8 smpc_out, const uint8 smpc_out_asserted) override;
 
@@ -68,7 +68,7 @@ class IODevice_Keyboard final : public IODevice
  int8 phase;
 };
 
-extern const IDIISG IODevice_Keyboard_US101_IDII;
+MDFN_HIDE extern const IDIISG IODevice_Keyboard_US101_IDII;
 }
 
 #endif
