@@ -86,9 +86,10 @@ static int32 SpriteResumeBase(const uint16* cmd_data)
  auto* fnptr = LineFuncTab[(bool)(FBCR & FBCR_DIE)][(TVMR & TVMR_8BPP) ? ((TVMR & TVMR_ROTATE) ? 2 : 1) : 0][(mode >> 6) & 0x1F][(mode & 0x8000) ? 8 : (mode & 0x7)];
  LineData.tffn = TexFetchTab[(mode >> 3) & 0x1F];
  //
+ // Don't merge e0 and e1 into a single array, keeping them separate is a workaround for gcc bug #113255
  //
- //
- EdgeStepper e[2] = { PrimData.e[0], PrimData.e[1] };
+ EdgeStepper e0 = PrimData.e[0];
+ EdgeStepper e1 = PrimData.e[1];
  VileTex big_t = PrimData.big_t;
  const uint32 tex_base = PrimData.tex_base;
  int32 iter = PrimData.iter;
@@ -105,8 +106,8 @@ static int32 SpriteResumeBase(const uint16* cmd_data)
  {
   do
   {
-   e[0].GetVertex<gourauden>(&LineData.p[0]);
-   e[1].GetVertex<gourauden>(&LineData.p[1]);
+   e0.GetVertex<gourauden>(&LineData.p[0]);
+   e1.GetVertex<gourauden>(&LineData.p[1]);
 
    LineData.tex_base = tex_base + big_t.PreStep();
    //
@@ -120,14 +121,14 @@ static int32 SpriteResumeBase(const uint16* cmd_data)
      break;
    }
 
-   e[0].Step<gourauden>();
-   e[1].Step<gourauden>();
+   e0.Step<gourauden>();
+   e1.Step<gourauden>();
   } while(MDFN_LIKELY(--iter >= 0 && ret < VDP1_SuspendResumeThreshold));
  }
  //
  //
- PrimData.e[0] = e[0];
- PrimData.e[1] = e[1];
+ PrimData.e[0] = e0;
+ PrimData.e[1] = e1;
  PrimData.big_t = big_t;
  PrimData.iter = iter;
 
