@@ -130,8 +130,19 @@ enum
  GSREG_CYCA0,
  GSREG_CYCA1 = GSREG_CYCA0 + 1,
  GSREG_CYCB0 = GSREG_CYCA0 + 2,
- GSREG_CYCB1 = GSREG_CYCA0 + 3
+ GSREG_CYCB1 = GSREG_CYCA0 + 3,
 
+ GSREG_RPMD,
+ GSREG_RPRCTL,
+ GSREG_KTCTL,
+ GSREG_KTAOF,
+ GSREG_OVPNRA,
+ GSREG_OVPNRB,
+ GSREG_RPTA,
+
+ GSREG_PRINA,
+ GSREG_PRINB,
+ GSREG_PRIR,
 };
 
 uint32 GetRegister(const unsigned id, char* const special, const uint32 special_len) MDFN_COLD;
