@@ -419,16 +419,17 @@ bool MDFNSS_StateAction(StateMem *sm, const unsigned load, const bool data_only,
    }
    else
    {
+    const size_t sname_len = strlen(sname);
     int64 data_start_pos;
     int64 end_pos;
     uint8 sname_tmp[32];
 
-    memset(sname_tmp, 0, sizeof(sname_tmp));
-    strncpy((char *)sname_tmp, sname, 32);
-
-    if(strlen(sname) > 32)
+    if(sname_len > 32)
      printf("Warning: section name is too long: %s\n", sname);
 
+    memset(sname_tmp, 0, sizeof(sname_tmp));
+    memcpy(sname_tmp, sname, std::min<size_t>(32, sname_len));
+
     st->write(sname_tmp, 32);
 
     st->put_LE<uint32>(0);                // We'll come back and write this later.
