@@ -2,7 +2,7 @@
 /* Mednafen - Multi-system Emulator                                           */
 /******************************************************************************/
 /* primitives.h:
-**  Copyright (C) 2013-2016 Mednafen Team
+**  Copyright (C) 2013-2020 Mednafen Team
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License
@@ -25,11 +25,18 @@
 namespace Mednafen
 {
 
-// Note: For simplicity, these functions do NOT perform surface dimension clipping.
+void MDFN_DrawFillRect(MDFN_Surface* surface, const MDFN_Rect& crect, int32 x, int32 y, uint32 w, uint32 h, uint32 color);
 
-void MDFN_DrawRect(MDFN_Surface *surface, uint32 x, uint32 y, uint32 w, uint32 h, uint32 border_color);
-void MDFN_DrawFillRect(MDFN_Surface *surface, uint32 x, uint32 y, uint32 w, uint32 h, uint32 border_color, uint32 fill_color);
-void MDFN_DrawFillRect(MDFN_Surface *surface, uint32 x, uint32 y, uint32 w, uint32 h, uint32 fill_color);
+enum class RectStyle
+{
+ Normal = 0,
+ Rounded
+};
+
+// Note: For simplicity, these functions do NOT perform surface dimension clipping:
+void MDFN_DrawRect(MDFN_Surface *surface, uint32 x, uint32 y, uint32 w, uint32 h, uint32 border_color, RectStyle style = RectStyle::Normal);
+void MDFN_DrawFillRect(MDFN_Surface *surface, uint32 x, uint32 y, uint32 w, uint32 h, uint32 border_color, uint32 fill_color, RectStyle style = RectStyle::Normal);
+void MDFN_DrawFillRect(MDFN_Surface *surface, uint32 x, uint32 y, uint32 w, uint32 h, uint32 fill_color, RectStyle style = RectStyle::Normal);
 void MDFN_DrawLine(MDFN_Surface *surface, int x0, int y0, int x1, int y1, uint32 color);
 
 }
